<?php
defined('ABSPATH') || die('');

/**
 * Class Linguise Configuration
 */
class LinguiseConfiguration
{
    /**
     * Languages names
     *
     * @var array
     */
    public $languages_names = array();

    /**
     * LinguiseConfiguration constructor.
     *
     * @param array $languages_names Languages names
     */
    public function __construct($languages_names)
    {
        if (!empty($languages_names)) {
            $this->languages_names = $languages_names;
        }
        add_action('admin_menu', array($this, 'registerMenuPage'));
        add_action('admin_head', array($this, 'adminHead'));
        add_action('admin_enqueue_scripts', array($this, 'enqueueScripts'));
    }



    /**
     * Function custom menu icon
     *
     * @return void
     */
    public function adminHead()
    {
        echo '<style>
    #toplevel_page_linguise .dashicons-before img {
            width: 25px;
            padding-top: 7px;
        }
  </style>';
    }

    /**
     * Register menu page
     *
     * @return void
     */
    public function registerMenuPage()
    {
        add_menu_page(
            'Linguise',
            'Linguise',
            'manage_options',
            'linguise',
            array($this, 'renderSettings'),
            LINGUISE_PLUGIN_URL . 'assets/images/linguise-logo.svg'
        );
    }

    /**
     *  Render settings
     *
     * @return void
     */
    public function renderSettings()
    {
        // check user capabilities
        if (!current_user_can('manage_options')) {
            return;
        }

        $errors = [];

        if (isset($_POST['linguise_options'])) {
            if (empty($_POST['main_settings'])
                || !wp_verify_nonce($_POST['main_settings'], 'linguise-settings')) {
                die();
            }

            $old_options =  linguiseGetOptions();
            $default_language = sanitize_key($_POST['linguise_options']['default_language']);

            $translate_languages = [];

            $token = sanitize_text_field($_POST['linguise_options']['token']);

            if ($old_options['token'] !== $token && $token !== '') {
                $args  = array(
                    'method'              => 'GET',
                    'headers'             => array('Referer' => site_url(), 'authorization' => $token)
                );

                $result =  wp_remote_get('https://api.linguise.com/api/config', $args);
                if (!is_wp_error($result) && isset($result['response']['code'])
                    && ($result['response']['code'] === 200) && !empty($result['body'])) {
                    $apiResponse = json_decode($result['body']);
                    if (!empty($apiResponse) && is_object($apiResponse)) {
                        $default_language = sanitize_key($apiResponse->data->language);
                        $translation_languages = $apiResponse->data->languages;
                        if (!empty($translation_languages)) {
                            foreach ($translation_languages as $translation_language) {
                                $translate_languages[] = sanitize_key($translation_language->code);
                            }
                        }
                    }
                } else {
                    if (!is_wp_error($result) && !empty($result['response']['code']) && $result['response']['code'] === 404) {
                        $errors[] = [
                            'type' => 'error',
                            'message' => sprintf(__('The API Key provided has been rejected, please make sure you use the right key associated with the domain %s', 'linguise'), site_url()),
                        ];
                    } else {
                        $errors[] = [
                            'type' => 'error',
                            'message' => __('Configuration has not been loaded from Linguise website. Please try again later or contact our support team if the problem persist.', 'linguise'),
                        ];
                    }
                    if (!empty($old_options['enabled_languages'])) {
                        $translate_languages = $old_options['enabled_languages'];
                    }
                }
            } else {
                if (!empty($_POST['enabled_languages_sortable'])) {
                    $lang_lists = explode(',', $_POST['enabled_languages_sortable']);
                } else {
                    $lang_lists = (!empty($_POST['linguise_options']['enabled_languages'])) ? $_POST['linguise_options']['enabled_languages'] : array();
                }

                if (!empty($lang_lists)) {
                    foreach ($lang_lists as $language) {
                        $translate_languages[] = sanitize_key($language);
                    }
                }
            }

            $pre_text = preg_replace('#<script(.*?)>(.*?)</script>#is', '', stripslashes($_POST['linguise_options']['pre_text']));
            $post_text = preg_replace('#<script(.*?)>(.*?)</script>#is', '', stripslashes($_POST['linguise_options']['post_text']));
            $alternate_link = $_POST['linguise_options']['alternate_link'] === '1' ? 1 : 0;
            $enable_flag = $_POST['linguise_options']['enable_flag'] === '1' ? 1 : 0;
            $enable_language_name = $_POST['linguise_options']['enable_language_name'] === '1' ? 1 : 0;
            $browser_redirect = $_POST['linguise_options']['browser_redirect'] === '1' ? 1 : 0;
            $cache_enabled = $_POST['linguise_options']['cache_enabled'] === '1' ? 1 : 0;
            $cache_max_size = isset($_POST['linguise_options']['cache_max_size']) ? (int)$_POST['linguise_options']['cache_max_size'] : 200;
            $linguise_options = array(
                'token' => $token,
                'default_language' => $default_language,
                'enabled_languages' => $translate_languages,
                'pre_text' => $pre_text,
                'post_text' => $post_text,
                'alternate_link' => $alternate_link,
                'enable_flag' => $enable_flag,
                'enable_language_name' => $enable_language_name,
                'custom_css' => isset($_POST['linguise_options']['custom_css']) ? $_POST['linguise_options']['custom_css'] : '',
                'flag_display_type' => isset($_POST['linguise_options']['flag_display_type']) ? $_POST['linguise_options']['flag_display_type'] : 'popup',
                'display_position' => isset($_POST['linguise_options']['display_position']) ? $_POST['linguise_options']['display_position'] : 'no',
                'language_name_display' => isset($_POST['linguise_options']['language_name_display']) ? $_POST['linguise_options']['language_name_display'] : 'en',
                'flag_shape' => isset($_POST['linguise_options']['flag_shape']) ? $_POST['linguise_options']['flag_shape'] : 'rounded',
                'flag_border_radius' => isset($_POST['linguise_options']['flag_border_radius']) ? (int)$_POST['linguise_options']['flag_border_radius'] : 0,
                'flag_width' => isset($_POST['linguise_options']['flag_width']) ? (int)$_POST['linguise_options']['flag_width'] : 24,
                'language_name_color' => isset($_POST['linguise_options']['language_name_color']) ? $_POST['linguise_options']['language_name_color'] : '#222',
                'language_name_hover_color' => isset($_POST['linguise_options']['language_name_hover_color']) ? $_POST['linguise_options']['language_name_hover_color'] : '#222',
                'flag_shadow_h' => isset($_POST['linguise_options']['flag_shadow_h']) ? (int)$_POST['linguise_options']['flag_shadow_h'] : 3,
                'flag_shadow_v' => isset($_POST['linguise_options']['flag_shadow_v']) ? (int)$_POST['linguise_options']['flag_shadow_v'] : 3,
                'flag_shadow_blur' => isset($_POST['linguise_options']['flag_shadow_blur']) ? (int)$_POST['linguise_options']['flag_shadow_blur'] : 6,
                'flag_shadow_spread' => isset($_POST['linguise_options']['flag_shadow_spread']) ? (int)$_POST['linguise_options']['flag_shadow_spread'] : 0,
                'flag_shadow_color' => isset($_POST['linguise_options']['flag_shadow_color']) ? $_POST['linguise_options']['flag_shadow_color'] : '#bfbfbf',
                'flag_hover_shadow_h' => isset($_POST['linguise_options']['flag_hover_shadow_h']) ? (int)$_POST['linguise_options']['flag_hover_shadow_h'] : 3,
                'flag_hover_shadow_v' => isset($_POST['linguise_options']['flag_hover_shadow_v']) ? (int)$_POST['linguise_options']['flag_hover_shadow_v'] : 3,
                'flag_hover_shadow_blur' => isset($_POST['linguise_options']['flag_hover_shadow_blur']) ? (int)$_POST['linguise_options']['flag_hover_shadow_blur'] : 6,
                'flag_hover_shadow_spread' => isset($_POST['linguise_options']['flag_hover_shadow_spread']) ? (int)$_POST['linguise_options']['flag_hover_shadow_spread'] : 0,
                'flag_hover_shadow_color' => isset($_POST['linguise_options']['flag_hover_shadow_color']) ? $_POST['linguise_options']['flag_hover_shadow_color'] : '#bfbfbf',
                'browser_redirect' => $browser_redirect,
                'cache_enabled' => $cache_enabled,
                'cache_max_size' => $cache_max_size
            );

            update_option('linguise_options', $linguise_options);

            if ($token !== '') {
                try {
                    $this->patchHtaccess();
                } catch (Exception $e) {
                    $errors[] = [
                        'type' => $e->getCode() === 0 ? 'warning' : 'error',
                        'message' => $e->getMessage(),
                    ];
                }
            }

            echo '<div class="linguise_saved_wrap"><span class="material-icons"> done </span> '. esc_html__('Linguise settings saved!', 'linguise') .'</div>';
        }
        require_once(LINGUISE_PLUGIN_PATH . DIRECTORY_SEPARATOR . 'src/admin/views' . DIRECTORY_SEPARATOR . 'view.php');
    }

    /**
     * Patch htaccess file
     *
     * @throws Exception With custom error message
     *
     * @return void
     */
    protected function patchHtaccess()
    {
        global $wp_filesystem;
        if (empty($wp_filesystem)) {
            require_once(ABSPATH . 'wp-admin/includes/file.php');
            WP_Filesystem();
        }

        // Save htaccess content
        $htaccess_path = ABSPATH . DIRECTORY_SEPARATOR . '.htaccess';
        if (!$wp_filesystem->exists($htaccess_path)) {
            throw new Exception(__('Htaccess file doesn\'t exist. This may be a problem is you are under an Apache server. Please check the documentation to finish the installation manually: <a href="https://www.linguise.com/documentation/linguise-installation/install-linguise-on-wordpress/" target="_blank">how to configure Linguise</a>', 'linguise'), 0);
        }

        $script_path = LINGUISE_PLUGIN_PATH . 'script.php';
        $script_path_parts = explode('/', trim($script_path, '/'));
        $abspath_parts = explode('/', trim(ABSPATH, '/'));

        $script_relative_path = array_slice($script_path_parts, count($abspath_parts));
        $script_relative_path = implode('/', $script_relative_path);

        $htaccess_content = $wp_filesystem->get_contents($htaccess_path);
        $htaccess_content_original = $htaccess_content;
        $htaccess_patched = false;
        if (strpos($htaccess_content, '#### LINGUISE DO NOT EDIT ####') !== false) {
            $htaccess_patched = true;
        }

        $content =
            '#### LINGUISE DO NOT EDIT ####' . PHP_EOL .
            '<IfModule mod_rewrite.c>' . PHP_EOL .
            '   RewriteEngine On' . PHP_EOL .
            '   RewriteRule ^(af|sq|am|ar|hy|az|bn|bs|bg|ca|zh-cn|hr|cs|da|nl|en|eo|et|fi|fr|de|el|gu|ht|ha|iw|hi|hmn|hu|is|ig|id|ga|it|ja|kn|kk|km|ko|ku|lo|lv|lt|lb|mk|mg|ms|ml|mt|mi|mr|mn|ne|no|ps|fa|pl|pt|pa|ro|ru|sm|sr|sd|sk|sl|es|su|sw|sv|tg|ta|te|th|tr|uk|ur|vi|cy|zz-zz)(?:$|/)(.*)$ ' . $script_relative_path . '?linguise_language=$1&original_url=$2 [L,QSA]' . PHP_EOL .
            '</IfModule>' . PHP_EOL .
            '#### LINGUISE DO NOT EDIT END ####' . PHP_EOL;

        if ($htaccess_patched) {
            // Replace previous version
            $htaccess_content = preg_replace('/#### LINGUISE DO NOT EDIT ####.*?#### LINGUISE DO NOT EDIT END ####/', $content, $htaccess_content);
        } else {
            // Add it at the beginning of the file
            $htaccess_content = $content . PHP_EOL . $htaccess_content;
        }

        if ($htaccess_content_original === $htaccess_content) {
            return;
        }

        if (!is_writable($htaccess_path)) {
            throw new Exception(__('Htaccess file is not writable, please make sure to allow the current script to update the .htaccess file to make linguise work as expected. You can also check our online documentation to read <a href="https://www.linguise.com/documentation/linguise-installation/install-linguise-on-wordpress/" target="_blank">how to configure Linguise</a>.', 'linguise'), 1);
        }

        // Only write if necessary
        if (!$wp_filesystem->put_contents($htaccess_path, $htaccess_content)) {
            throw new Exception(__('Failed to write to htaccess file, please make sure to allow the current script to update the .htaccess file to make linguise work as expected. You can also check our online documentation to read <a href="https://www.linguise.com/documentation/linguise-installation/install-linguise-on-wordpress/" target="_blank">how to configure Linguise</a>.', 'linguise'), 2);
        }
    }

    /**
     * Enqueue scripts
     *
     * @return void
     */
    public function enqueueScripts()
    {
        global $current_screen;
        if (!empty($current_screen) && $current_screen->id === 'toplevel_page_linguise') {
            wp_enqueue_script('jquery-ui-sortable');
            wp_enqueue_style('wp-color-picker');
            wp_enqueue_script('wp-color-picker');
            wp_enqueue_script(
                'linguise_chosen_sortable_script',
                LINGUISE_PLUGIN_URL . 'assets/js/jquery-chosen-sortable.min.js',
                array('jquery'),
                LINGUISE_VERSION,
                true
            );
            wp_enqueue_script(
                'linguise_admin_script',
                LINGUISE_PLUGIN_URL . 'assets/js/admin.bundle.js',
                array('jquery'),
                LINGUISE_VERSION,
                true
            );
            wp_localize_script('linguise_admin_script', 'linguise', array(
                'linguise_nonce' => wp_create_nonce('linguise_nonce'),
                'ajaxurl'               => admin_url('admin-ajax.php')
            ));
            wp_enqueue_style(
                'linguise_admin_script',
                LINGUISE_PLUGIN_URL . 'assets/css/admin.bundle.css',
                array(),
                LINGUISE_VERSION
            );
            wp_enqueue_style('linguise_switcher', LINGUISE_PLUGIN_URL . '/assets/css/front.bundle.css', array(), LINGUISE_VERSION);
        }
    }
}

new LinguiseConfiguration($languages_names);
