<?php

namespace Linguise\Script\Core;

use Linguise\Script\Core\Request;

defined('LINGUISE_SCRIPT_TRANSLATION') or die();

class Cache
{
    /**
     * @var null|Request
     */
    private static $_instance = null;

    /**
     * Hash of the original content retrieved from untranslated page
     *
     * @var null|string
     */
    private $_hash = null;

    /**
     * @var null|string
     */
    private $_content = null;

    /**
     * @var null|string
     */
    private $_language = null;

    /**
     * @var null
     */
    private $_curl_multi = null;

    /**
     * Retrieve singleton instance
     *
     * @return Request|null
     */
    public static function getInstance() {

        if(is_null(self::$_instance)) {
            self::$_instance = new Cache();
        }

        return self::$_instance;
    }

    public function getPath() {
        return Configuration::getInstance()->get('data_dir') . DIRECTORY_SEPARATOR . 'cache' . DIRECTORY_SEPARATOR;
    }

    public function serve() {
        $response = Response::getInstance();

        $content = $response->getContent();

        if (!$content) {
            return false;
        }

        $this->_hash = md5(json_encode(array(
            'content' => $content,
            'url' => Request::getInstance()->getRequestedUrl()
        )));

        $language = Request::getInstance()->getLanguage();

        if (!preg_match('/^[a-z]{2,3}(?:-[a-z]{2})?$/', $language)) {
            // Some characters are not allowed skip
            return false;
        }

        $this->_language = $language;

        if (!$this->load()) {
            return false;
        }

        $response->setContent($this->_content);

        $response->end();
    }

    protected function load() {
        $cache_file = Configuration::getInstance()->get('data_dir') . DIRECTORY_SEPARATOR . 'cache' . DIRECTORY_SEPARATOR . $this->_language . '_' . $this->_hash . '.php';

        if (!file_exists($cache_file)) {
            return false;
        }

        $content = file_get_contents($cache_file);

        // Update cache file modified time
        touch($cache_file);

        // Remove php head
        $this->_content = substr($content, 15);

        return true;
    }

    public function save() {
        if (!$this->_hash || !$this->_language) {
            return false;
        }

        $response = Response::getInstance();

        $content = $response->getContent();

        if (!$content) {
            return false;
        }

        $cache_path = $this->getPath();
        $cache_file = $cache_path . $this->_language . '_' . $this->_hash . '.php';

        if (!file_exists($cache_path)) {
            mkdir($cache_path);
        }

        file_put_contents($cache_file, '<?php die(); ?>' . $content);

        return true;
    }

    public function shoudBeCleared() {
        $cache_info_file = $this->getPath() . 'clear.txt';
        if (file_exists($cache_info_file) && (int)file_get_contents($cache_info_file) + Configuration::getInstance()->get('cache_time_check') > time()) {
            return false;
        }
        return true;
    }

    public function clear() {
        $cache_path = $this->getPath();

        $files = glob($cache_path . '*.php');

        usort($files, function($x, $y) {
            return filemtime($x) < filemtime($y);
        });

        $max_size = Configuration::getInstance()->get('cache_max_size') * 1024 * 1024;
        $total_size = 0;
        $total_cleared = 0;
        foreach($files as $file) {
            if ($total_size > $max_size) {
                unlink($file);
                continue;
            }
            $size = filesize($file);
            $total_cleared += $size;
            $total_size += $size;
        }

        file_put_contents($cache_path . 'clear.txt', time());

        $response = Response::getInstance();
        $response->setContent('Cleared cache: ' . (int)($total_cleared/1000) . 'kb');
        $response->end();
    }

    public function doClearRequest() {
        if (!Configuration::getInstance()->get('cache_enabled') || !$this->shoudBeCleared()) {
            return;
        }

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_NOBODY, true); // SET HEAD type
        curl_setopt($ch, CURLOPT_URL, Request::getInstance()->getBaseUrl() . '/zz-zz/?linguise_action=clear-cache');
        if (Configuration::getInstance()->get('server_ip') !== null) {
            curl_setopt($ch, CURLOPT_CONNECT_TO, [Request::getInstance()->getHostname() . ':' . Configuration::getInstance()->get('server_port') . ':' . Configuration::getInstance()->get('server_ip') . ':' . Configuration::getInstance()->get('server_port')]); // fixme: only available from php 7.0
        }
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 2);
        curl_setopt($ch, CURLOPT_TIMEOUT, 2);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        if (Configuration::getInstance()->get('dl_certificates') === true) {
            curl_setopt($ch, CURLOPT_CAINFO, Certificates::getPath());
        }

        $this->_curl_multi = curl_multi_init();
        curl_multi_add_handle($this->_curl_multi, $ch);
        curl_multi_exec($this->_curl_multi, $still_running);
    }

    public function waitRequest()
    {
        if (!$this->_curl_multi) {
            return;
        }

        do {
            $status = curl_multi_exec($this->_curl_multi, $active);
            if ($active) {
                // Wait a short time for more activity
                curl_multi_select($this->_curl_multi);
            }
        } while ($active && $status === CURLM_OK);

    }

}